#!/usr/bin/python3

import sys
from sympy import *
init_printing(use_unicode=True)

# uz pomoc sympy biblioteke mozemo da racunamo i analiticke probleme

# prvo definisemo simbole koje cemo koristiti u primerima
x = Symbol('x')
y = Symbol('y')
z = Symbol('z')

# sympy podrzava rad sa limesima pomocu funkcije limit
# potrebno je samo da definisemo funkciju i kazemo cemu tezi promenljiva

print(limit(sin(x)/x, x, 0))
print(limit(x, x, oo))
print(limit(1/x, x, oo))
print(limit(x**x, x, 0))

# pored izracunavanja limesa, sympy ume da racuna i izvode funkcija pomocu funkcije diff
print(diff(sin(x), x))
print(diff(2*sin(x/3), x))
print(diff(tan(x), x))

# mozemo da probamo i po definicija da izracunamo izvod
print(limit((tan(x+y) - tan(x))/y, y, 0))

# pored jednostrukih izvoda, moguce je raditi i visetruke izvode tako sto se navede
# treci argument

# prvi izvod
print(diff(2*sin(x/3), x, 1))
# drugi izvod
print(diff(2*sin(x/3), x, 2))
# treci izvod
print(diff(2*sin(x/3), x, 3))

# sympy ume da racuna parcijalne izvode funkcija vise promenljivih
# pri cemu se izvodi rade onim redom kojim su navedeni u listi argumenata
expr = exp(x*y*z)
# stampamo parcijalni izovd po x, y^2 i z^4
print(diff(expr, x, y, y, z, z, z, z))
# sto je ekvivalentno sa
print(diff(expr, x, y, 2, z, 4))

# pored izvoda, integrali su takodje direktno podrzani u sympy bilbioteci. 
# za razliku od ranije pomenutih metoda integracije u scipy, integracija u sympy je simbolicka
# sto znaci da je rezultat integraljenja analiticki oblik resenja
print(integrate(cos(x), x))
print(integrate(log(x), x))
print(integrate(1/(1+x**2), x))
# ali to ne znaci da ne mozemo da resavamo i odredjene integrale
print(integrate(exp(-x), (x, 0, oo)))
# iako broj, resenje je simbolicko, tj. apsolutno tacno

# pored jednostrukih integrala, moguce je izracunavati i visestruke integrale
print(integrate(exp(-x**2 - y**2), (x, -oo, oo), (y, -oo, oo)))

# u slucaju da integrate ne moze da nadje simbolicko resenje integrala, povratna vrednost ce biti Integral.
# u tom slucaju ili treba rucno uvesti neku smenu ili resiti integral numericki

# razvijanje funkcija u tejlorov red je takodje dostupno u sympy paketu

# definisemo funkciju
expr = exp(sin(x))

# razivjamo funkciju oko 0 u red 4. stepena
print(expr.series(x, 0, 4))



