#!/usr/bin/python3

import sys
# ukljucujemo biblioteku numpy
import numpy as np

# kreiramo dvodimenzioni niz dimenzije 3x4
a = np.array([[1,2,3,4],[5,6,7,8],[9,10,11,12]])

# kao i u slucaju stringova i obicnih nizova i sa numpy nizovima 
# mozemo da koristimo 'slicing', odnosno izdvajanje podnizova

# ukoliko zelimo da izdvojimo podniz koga cine prva i druga kolona 
# i sve vrste manje od dva treba da napisemo sledeci izraz
b = a[:2, 1:3]

# prilikom indeksiranja na ovaj nacin, treba biti posebno 
# pazljiv kada su granice u pitanju. opsti oblik indeksiranja 
# je [min:max], pri cemu se min granica tretira kao >=, a max 
# granica kao <. Jos jedna vazna napomena je da mora da vazi 
# min <= max, jer u suprotnom izraz nece moci da se evaluira. 
# npr. zapis [:2] znaci od pocetka do indeksa strogo manjeg od 2. 
# 		-> izdvajaju se samo 0 i 1
# npr. zapis [1:3] znaci od indeksa 1 do indeksa strogo manjeg od 3. 
# 		->izdvajaju se 1 i 2

# kada ovaj zapis sklopimo u jedno indeksiranje oblika [:2, 1:3], 
# ono se tumaci kao izvlacenje podniza sacinjenog od elemenata iz 
# 0. i 1. vrste koji se nalaze u 1. i 2. koloni.

# prikazujemo dobijeni podniz
print(b)

# stampamo element u nizu a koji odgovara prvom elementu niza b
print(a[0,1])
# modifikujemo prvi element niza b
b[0,0] = 77
# stampamo ponovo prvi element niza a koji odgovara nizu b
print(a[0,1])

# BITNO: izdvajanje podniza na ovaj nacin ne kreira novi niz, 
# vec samo kreira novi pogled na vec postojeci niz. 
# Ovakvo ponasanje za sobom povlaci da bilo koja izmena izdvojenog 
# podniza direktno menja polazni niz.
# Ako ste koritili Matlab ranije i navikli ste na njegov nacin rada,
# treba da budete posebno oprezni kod izvrsavanja ovakvih naredbi,
# jer se ponasaju skroz drugacije

# pored klasicnog indeksiranja numpy, podrzava i indeksiranje nizova 
# drugim nizovima

# kreiramo novi niz
a = np.array([[1,2], [3, 4], [5, 6]])

# pristupamo elementima niza a, pomocu niza indeksa
print(a[[0, 1, 2], [0, 1, 0]])

# zapis [[0, 1, 2], [0, 1, 0]] se tumaci kao skup uredjenih parova 
# indeksa [0,0], [1,1], [2,0]
# pa je ovo u stvari izdvajanje elemenata a[0,0], a[1,1], a[2,0] 
# i njihovom smestanju u novi niz [a[0,0], a[1,1], a[2,0]]

# zapis iz prethodne naredbe je ekvivalentan zapisu
print(np.array([a[0, 0], a[1, 1], a[2, 0]]))

# na ovaj nacin mozemo vise puta da koristimo isti element iz niza
print(a[[0, 0], [1, 1]])
# sto je ekvivalentno sa
print(np.array([a[0, 1], a[0, 1]]))

# kreiramo novi niz
a = np.array([[1,2,3],[4,5,6],[7,8,9],[10,11,12]])

# kreiramo niz indeksa
b = np.array([0,2,0,1])

# ako hocemo da prikazemo elemente iz svake vrste pomocu niza indeksa b, 
# to mozemo da postignemo na sledeci nacin
print(a[np.arange(4), b])

# ako hocemo da izmenimo vrednost tih elemenata, to postizemo na slican nacin
a[np.arange(4), b] += 10

# stampamo izmenjeni niz
print(a)

# pored klasicnog numerickog indeksiranja, numpy nizovi podrzavaju i bulovsko indeksiranje

# kreiramo novi niz
a = np.array([[1,2], [3, 4], [5, 6]])

# kreiramo niz bulovskih indeksa koji nam govore na kojim mestima se u nizu a
# nalaze brojevi veci od 2
bool_idx = (a > 2)

# stampamo dobijeni niz indeksa
print(bool_idx)

# izdvajamo podniz elemenata koji zadovoljavaju uslov a > 2
print(a[bool_idx])

# sve ovo moguce je zapisati i u jednoj naredbi
print(a[a > 2])     
