#!/usr/bin/python3

import sys
# da bismo koristili reduce moramo da ucitamo odgovarajucu biblioteku
from functools import reduce

# anonimne funkcije i lambda izrazi
# lamdba izrazi su mehanizam kojim se kreiraju mali parcici koda koji
# su potrebni samo u tom delu programa i nikada vise. Imajuci u vidu njihovu ogranicenu upotrebu
# besmisleno je davati im imena, te se stoga nazivaju jos i anonimne funkcije

# opsti oblik definisanja lambda izraza je sledeci
# lambda lista_argumenata : izraz

# pravimo jednostavnu anonimnu funkciju koja racuna zbir dva broja
# lambda izrazu cemo dodeliti ime koje cemo koristiti kasnije, iako to nije u 
# duhu sa upotrebom anonimnih funkcija
sum = lambda x, y: x + y

# ukoliko hocemo da upotrebimo ovaj lambda izraz da bismo nasli zbir dva broja
# dovoljno je da napisemo sledece
zbir = sum(3,4)
# prikazujemo  dobijeni rezultat
print(zbir)

# prethodni primer je vestacki i lambda izrazi se ne koriste tako u realnim programima
# lambda izrazi se najcesce koriste u paru sa funkcijama map, filter i reduce

# funkcija map se koristi ako zelimo da primenimo istu operaciju na svaki element kolekcija
# na primer, zelimo da kvadriramo svaki element liste
 
# kreiramo listu
L = list(range(1, 10))
# stampamo kreiranu listu
print(L)

# kvadriramo svaki element liste
kvadrirani = list(map(lambda x: x**2, L))
# stampamo novu listu kvadriranih elemenata
print(kvadrirani)

# uvecavamo svaki element liste za 10
uvecani = list(map(lambda x: x + 10, kvadrirani))
# prikazujemo elemente
print(uvecani)

# definisemo listu temperatura
temperature = (36.5, 37.1, 37.5, 38.3, 39.7)
# prikazujemo temperature
print(temperature)
# pomocu lambda izraza prebacujemo temperature u farenhajte
farenhajti = list(map(lambda x: (9.0/5.0)*x + 32, temperature))
# prikazujemo rezultat
print(farenhajti)
# pomocu lambda izraza vracamo temperature u celzijuse
celzijusi = list(map(lambda x: (5.0/9.0)*(x - 32), farenhajti))
# prikazujemo rezultat
print(celzijusi)

# funkciju map i lambda izraze je moguce primeniti na vise od jedne liste
# jedini uslov je da liste budu jednake duzine

# definisemo tri liste
a = [1, 2, 3, 4]
b = [17, 12, 11, 10]
c = [-1, -4, 5, 9]
# izracunavamo zbir prve dve liste
zbir_a_b = list(map(lambda x,y:x+y, a,b))
# stampamo dobijeni rezultat
print(zbir_a_b)

# izracunavamo zbir sve tri liste
zbir_tri = list(map(lambda x,y,z:x+y+z, a,b,c))
# stampamo dobijeni rezultat
print(zbir_tri)

# izracunavamo malo slozeniji izraz
lista1 = list(map(lambda x,y,z : 2.5*x + 2*y - z, a,b,c))
# stampamo dobijeni rezultat
print(lista1)

# funkcija filter se koristi kada zelimo da izdvojimo elemente kolekcije
# koji zadovoljavaju neki uslov
# uslov se zadaje kao lambda izraz

# kreiramo listu prirodnih brojeva
L = list(range(1,20))
# stampamo kreiranu listu
print(L)

# iz lista iszdvajamo samo neparne elemente
neparni = list(filter(lambda x: x%2, L))
# stampamo dobijeni rezultat
print(neparni)

# iz list izdvajemo samo parne elemente
parni = list(filter(lambda x: x%2 == 0, L))
# stampamo dobijeni rezultat
print(parni)

# stampanje prostih brojeva
brojevi = list(range(2,50))
# filtriramo listu (Eratostenovo sito)
for i in range(2,8):
	brojevi = list(filter(lambda x: x == i or x % i, brojevi))

# algoritam kaze sledece:
# ostavi element u listi ako je jednak i ili ako podeljen sa i daje ostatak razlicit od 0
# u suprotnom, ukloni ga iz liste

# stampamo dobijeni rezultat
print(list(brojevi))

# funkcija reduce se koristi kada zelimo da svedemo listu na neku skalarnu vrednost
# reduce radi tako sto sto kontinualno primenjuje funkciju na glavu i rep liste.

# ako zamislimo da imamo listu s oblika [s1, s2 ,s3, s4] i da zelimo da je redukujemo po
# nekoj funkciji f
# sematski, primena funkcije f na redukovanje liste s bi tekla sledecim tokom

# prvo se izracunava vrednost funkcije za prva dva elementa
# f1 = f(s1, s2)
# a zatim se ta dobijena vrednost ukljucuje u izracunavanje sa sledecim elementom liste
# f2 = f(f1, s3) = f(f(s1, s2), s3)
# zatim se novi rezultat f2 ukljucuje u dalje izracunavanje
# f3 = f(f2, s4) = f(f(f(s1,s2),s3), s4)
# cim iskoristimo poslednji element, tada funkcija reduce prestaje sa radom

# imajuci ovo u vidu, mozemo da probamo da izracunam zbir elemenata u listi koriscenjem
# funkcije reduce

# kreiramo listu
S = list(range(1,100))
# da bismo napravili lambda izraz na pravi nacin, moramo da pazljivo definisemo ono
# sta zelimo da postignemo imajuci u vidu kako radi reduce

# reduce u svakom koraku primenjuje funkciju na stari zbir uzimajuci u obzir sledeci
# element u listi, pa bi lamdba izraz koji to opisuje treba da izgleda na sledeci nacin
# lambda x,y: x+y
# x je stari zbir, tj. zbir elemenata do i-tog
# y je taj i-ti element do kog smo stigli
# i rezultat lamdba izraza je zbir x+y, odnosno na tekuci zbir smo dodali novi
# sledeci element liste

# stampamo dobijeni zbir
print(reduce(lambda x,y:x+y, S))

# na slican nacin mozemo da probamo da nadjemo najveci element u listi

# definisemo lambda izraz koji opisuje veci od dva elementa
f = lambda a,b: a if (a>b) else b
# stampamo dobijeni rezultat
print(reduce(f, [47,11,42,102,13]))

# na osnovu primera sa zbirom brojeva lako mozemo da izracunamo proizvod, tj. faktorijel
print(reduce(lambda x,y:x*y, S))

# verovatnoca da izvucemo sedmicu na lotou
p = (1.0*reduce(lambda x,y:x*y, range(1,8)))/reduce(lambda x,y:x*y, range(43,50))
print(p)
print(1/p)

