#!/usr/bin/python3

import numpy as np
# da bismo crtali grafike u pajtonu, potrebna nam je 
# biblioteka matplotlib
from matplotlib import pyplot as plt

# sve sto smo do sada crtali je prikazivalo jedan crtez na jednoj slici
# ponekad, zelimo da prikazemo vise crteza na jednoj figuri radi uporedne analize

# da bismo to postigli, treba da koristimo funkciju subplot
# kojom figuru delimo pna matricu figura

# na primer, zelimo da prikazemo sve osnovne trigonometrijske funkcije na jednoj figuri
# crtez jedan treba da prikaze funkciju sinus,
# crtez dva treba da prikaze funkciju kosinus,
# crtez tri treba da prikaze eksponencijalnu funkciju
# crtez cetiri treba da prikaze logaritamsku funkciju

# kao i u prethodnom primeru, prvo pravimo podelu intervala

# podelu kreiramo ranije pomenutom funkcijom linspace iz paketa numpy
X = np.linspace(-np.pi, np.pi, 256, endpoint=True)
# zatim izracunavamo vrednosti funkcije u tackama podele
C, S= np.cos(X), np.sin(X)

# racunamo novu podelu za druge dve funkcije
XX = np.linspace(0.01, 10, 1000, endpoint=True)
CT = np.log(XX)

XXX = np.linspace(-10, 5, 1000, endpoint=True)
T = np.exp(XXX)

# podesavamo velicinu figure
plt.figure(figsize=(8,6))

# trebaju nam cetiri crteza na istog figuri i organizovacemo ih kao matricu 2x2
# polja u matrici se redom obelezavaju brojevima pocevsi od gornjeg levog polja
# u smeru s desna na levo.
# U slucaju matrice 2x2, numeracija polja bi bila sledeca
# 1 2
# 3 4


# u prvom polju zelimo da nacrtamo grafik funkcije sinus
# da bismo proglasili prvo polje kao aktivno, moramo da koristimo komandu subplot
plt.subplot(2,2,1)
# sva formatiranja koja navedemo u nastavku odnosice se samo na prvo polje na crtezu

# crtamo grafik
plt.plot(X, S, color="blue", linewidth=2.5, linestyle="-", label="sinus")
# ofarbamo pozitivni deo na grafiku
plt.fill_between(X, 0, S, S > 0, color='blue', alpha=.25)
# ofarbamo negativni deo na grafiku
plt.fill_between(X, S, 0, S < 0, color='red',  alpha=.25)

# postavljamo granice za prikazivanje grafika po x osi
plt.xlim(-4.0, 4.0)
# postavljamo granice za prikazivanje grafika po x osi
plt.ylim(-1.0, 1.0)
# obelezavamo tacaka na x osi
plt.xticks(np.linspace(-4,4,9, endpoint=True))
# obelezavamo tacaka na y osi
plt.yticks(np.linspace(-1,1,5,endpoint=True))
# prikazujemo legendu uz eksplicitno navodjenje lokacije
plt.legend(loc='upper left')
# prikazujemo mrezu
plt.grid()
# prikazujemo i ose
plt.axvline(0)
plt.axhline(0)

# na drugom polju crtamo funkciju kosinus

# proglsavamo drugo polje kao aktivno
plt.subplot(2,2,2)
# sva formatiranja koja navedemo u nastavku odnosice se samo na prvo polje na crtezu

# crtamo grafik
plt.plot(X, C, color="red", linewidth=2.5, linestyle="-", label="kosinus")
# postavljamo granice za prikazivanje grafika po x osi
plt.xlim(-4.0, 4.0)
# postavljamo granice za prikazivanje grafika po x osi
plt.ylim(-1.0, 1.0)
# obelezavamo tacaka na x osi
plt.xticks(np.linspace(-4,4,9, endpoint=True))
# obelezavamo tacaka na y osi
plt.yticks(np.linspace(-1,1,5,endpoint=True))
# prikazujemo legendu uz eksplicitno navodjenje lokacije
plt.legend(loc='upper left')
# prikazujemo mrezu
plt.grid()
# prikazujemo i ose
plt.axvline(0)
plt.axhline(0)

# na trecem polju crtamo funkciju eksponencijalnu funkciju

# proglsavamo drugo polje kao aktivno
plt.subplot(2,2,3)
# sva formatiranja koja navedemo u nastavku odnosice se samo na prvo polje na crtezu

# crtamo grafik
plt.plot(XXX, T, color="green", linewidth=2.5, linestyle="-", label="exp")

# postavljamo granice za prikazivanje grafika po x osi
plt.xlim(-10, 5)
# postavljamo granice za prikazivanje grafika po x osi
plt.ylim(-0.5, 50)
# obelezavamo tacaka na x osi
plt.xticks(np.linspace(-10, 5, 5, endpoint=True))
# obelezavamo tacaka na y osi
plt.yticks(np.linspace(--05, 50, 10,endpoint=True))
# prikazujemo legendu uz eksplicitno navodjenje lokacije
plt.legend(loc='upper left')
# prikazujemo mrezu
plt.grid()
# prikazujemo i ose
plt.axvline(0)
plt.axhline(0)

# na trecem polju crtamo logaritamsku funkciju 

# proglsavamo drugo polje kao aktivno
plt.subplot(2,2,4)
# sva formatiranja koja navedemo u nastavku odnosice se samo na prvo polje na crtezu

# crtamo grafik
plt.plot(XX, CT, color="magenta", linewidth=2.5, linestyle="-", label="logaritam")
# postavljamo granice za prikazivanje grafika po x osi
plt.xlim(0, 10.0)
# postavljamo granice za prikazivanje grafika po x osi
plt.ylim(-5.0, 5.0)
# obelezavamo tacaka na x osi
plt.xticks(np.linspace(0,10,11, endpoint=True))
# obelezavamo tacaka na y osi
plt.yticks(np.linspace(-5,5,11,endpoint=True))
# prikazujemo legendu uz eksplicitno navodjenje lokacije
plt.legend(loc='upper left')
# prikazujemo mrezu
plt.grid()
# prikazujemo i ose
plt.axvline(0)
plt.axhline(0)

# na kraju podesavamo zeljeni razmak izmedju polja na crtezu
plt.tight_layout()
# i prikazujemo crtez
plt.show()

