#!/usr/bin/python3

import numpy as np
# da bismo crtali grafike u pajtonu, potrebna nam je 
# biblioteka matplotlib
from matplotlib import pyplot as plt
# za crtanje 3d grafike moramo treba nam funkcija Axes3d iz
# paketa mpl_toolkits.mplot3d
from mpl_toolkits.mplot3d import Axes3D

# pored klasicnih grafika, biblioteka matplotlib podrzava i bar grafike

# pravimo dva niza slucajnih brojeva
n = 12;
X = np.arange(n);
Y1 = (1-X/float(n))*np.random.uniform(0.5,1.0,n)
Y2 = (1-X/float(n))*np.random.uniform(0.5,1.0,n)

# startujemo novu figuru
plt.figure(num = 1, figsize = (6,4))

# postavljamo novu osu
plt.axes([0.025, 0.025, 0.95, 0.95])
# crtamo oba niza sa razlicitim znacima
plt.bar(X, +Y1, facecolor='#9999ff', edgecolor='white')
plt.bar(X, -Y2, facecolor='#ff9999', edgecolor='white')

# postavljamo natpise iznad svakog pravougaonika
for x, y in zip(X, Y1):
    plt.text(x + 0.4, y + 0.05, '%.2f' % y, ha='center', va= 'bottom')

# postavljamo natpise ispod svakog pravougaonika	
for x, y in zip(X, Y2):
    plt.text(x + 0.4, -y - 0.05, '%.2f' % y, ha='center', va= 'top')

# postavljamo granice i obelezja na osama
plt.xlim(-.5, n)
plt.xticks(())
plt.ylim(-1.25, 1.25)
plt.yticks(())	

# prikazujemo crtez
plt.show()


# za prikazivanje uredjenih parova tacaka u ravni koristi se scatter plot
# scatter plot moze da se koristi i za prikazivanje krivih drugog reda

# kreiramo uredjene parove tacaka
n = 1024
X = np.random.normal(0, 1, n)
Y = np.random.normal(0, 1, n)
# boju tacke cemo da odredimo njenim uglom
T = np.arctan2(Y, X)

# startujemo novu figuru
plt.figure(num = 2, figsize = (6,4))

# definisemo novu osu na crtezu
plt.axes([0.025, 0.025, 0.95, 0.95])

# prikazujemo uredjene parove tacaka
plt.scatter(X, Y, s=75, c=T, alpha=.5)

# postavljamo granice i obelezja na osama
plt.xlim(-1.5, 1.5)
plt.xticks(())
plt.ylim(-1.5, 1.5)
plt.yticks(())

# prikazujemo crtez
plt.show()

# matplotlib ume da crta i 3d grafike
# da bismo crtali 3d graike potreban nam je objekat Axes3d iz paketa mpl_toolkits.mplot3d

# kreiramo novu figuru i pamtimo njenu referencu
fig = plt.figure(num = 3)
# definise ose na figuri
ax = Axes3D(fig)
# kreiramo podelu po x osi
X = np.arange(-4, 4, 0.25)
# kreiramo podelu po y osi
Y = np.arange(-4, 4, 0.25)
# na osnovu kreirane podele kreiramo mrezu koju cemo koristiti za
# crtanje
X, Y = np.meshgrid(X, Y)
# racunamo pomocnu funkciju
R = np.sqrt(X ** 2 + Y ** 2)
# racunamo vrednosti funkcije sinus u tackama podele
Z = np.sin(R)

# crtamo povrs
ax.plot_surface(X, Y, Z, rstride=1, cstride=1, cmap=plt.cm.hot)
# crtamo konture u XY ravni
ax.contourf(X, Y, Z, zdir='z', offset=-2, cmap=plt.cm.hot)
# postavljamo limit za prikazivanje grafika po z osi
ax.set_zlim(-2, 2)

# prikazujemo crtez
plt.show()
	
	
